local _G = getfenv(0)
local VERSION = tonumber(("$Revision: 42216 $"):match("%d+"))
local L = AceLibrary("AceLocale-2.2"):new("AuldLangSyne_Fu")

local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local BC = AceLibrary("Babble-Class-2.2")

local AuldLangSyne = AuldLangSyne
local Fu = AuldLangSyne:NewModule("Fu", "AceHook-2.1", "AceEvent-2.0", "FuBarPlugin-2.0")
if AuldLangSyne.revision < VERSION then
	AuldLangSyne.version = "r" .. VERSION
	AuldLangSyne.revision = VERSION
	AuldLangSyne.date = ("$Date: 2007-06-29 07:18:28 -0400 (Fri, 29 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local new, del, newHash, newSet, deepCopy, deepDel, clear = AuldLangSyne.new, AuldLangSyne.del, AuldLangSyne.newHash, AuldLangSyne.newSet, AuldLangSyne.deepCopy, AuldLangSyne.deepDel, AuldLangSyne.clear

local friends, guild, levels, classes, areas, status, connected, ranks, rankindex, gnotes, gonotes, party
local numFriends, numFriendsOnline, numGuild, numGuildOnline = 0, 0, 0, 0

Fu.name = "AuldLangSyneFu"
Fu.title = "AuldLangSyneFu"
Fu.clickableTooltip = true
Fu.hasIcon = "Interface\\FriendsFrame\\FriendsFrameScrollIcon"
Fu.hideWithoutStandby = true
Fu.OnMenuRequest = AuldLangSyne.menu

function Fu:OnInitialize(name)
	self.db = AuldLangSyne:AcquireDBNamespace("Fu")
	AuldLangSyne:RegisterDefaults("Fu", "profile", {
		friends = {
			show = true,
			sort = 'name',
			offline = false,
			columns = {
				name = true,
				level = true,
				class = false,
				zone = true,
				status = true,
				note = true,
			},
			colors = {
				name = 'class',
				level = 'level',
				class = 'class',
				zone = 'none',
			},
		},
		guild = {
			show = true,
			sort = 'name',
			offline = false,
			motd = true,
			columns = {
				name = true,
				level = true,
				class = false,
				zone = true,
				status = true,
				rank = true,
				note = true,
			},
			colors = {
				name = 'class',
				level = 'level',
				class = 'class',
				zone = 'none',
				rank = 'none',
			},
		},
	})
	
	local function makeGetter(kind, option)
		return function(key) return self.db.profile[kind][option][key] end
	end
	local function makeSetter(kind, option)
		return function(key, t) self.db.profile[kind][option][key] = t; self:Update() end
	end
	local function makeColumnsGroup(kind)
		group = {
			type = "group",
			name = "Columns", desc = "Columns",
			pass = true,
			get = makeGetter(kind, 'columns'),
			set = makeSetter(kind, 'columns'),
			args = {},
		}
		for column in pairs(self.db.profile[kind].columns) do
			group.args[column] = {type="toggle", name=column, desc=column,}
		end
		return group
	end
	local colorChoices = {'class', 'level', 'none'}
	local function makeColorsGroup(kind)
		group = {
			type = "group",
			name = "Colors", desc = "Colors",
			pass = true,
			get = makeGetter(kind, 'colors'),
			set = makeSetter(kind, 'colors'),
			args = {},
		}
		for column in pairs(self.db.profile[kind].colors) do
			group.args[column] = {type='text', name=column, desc=column, validate=colorChoices,}
		end
		return group
	end
	self.menu = {
		handler = Fu,
		name = "FuBar", type = "group",
		desc = "FuBar plugin options",
		args = {
			friends = {
				type = "group",
				name = "Friends", desc = "Friends",
				args = {
					show = {
						type = 'toggle',
						name = "Show", desc = "Show",
						get = function() return self.db.profile.friends.show end,
						set = function(t) self.db.profile.friends.show = t; self:Update() end,
						order = 1,
					},
					offline = {
						type = 'toggle',
						name = 'Offline', desc = "Offline",
						get = function() return self.db.profile.friends.offline end,
						set = function(t) self.db.profile.friends.offline = t; self:UpdateTooltip() end,
					},
					sort = {
						type = 'text',
						name = 'Sort', desc = 'Sort',
						get = function() return self.db.profile.friends.sort end,
						set = function(t) self.db.profile.friends.sort = t; self:Update() end,
						validate = {'name', 'level', 'online', 'zone', 'class'},
					},
					columns = makeColumnsGroup('friends'),
					colors = makeColorsGroup('friends'),
				},
			},
			guild = {
				type = "group",
				name = "Guild", desc = "Guild",
				args = {
					show = {
						type = 'toggle',
						name = "Show", desc = "Show",
						get = function() return self.db.profile.guild.show end,
						set = function(t) self.db.profile.guild.show = t; self:Update() end,
						order = 1,
					},
					offline = {
						type = 'toggle',
						name = 'Offline', desc = "Offline",
						get = function() return self.db.profile.guild.offline end,
						set = function(t) self.db.profile.guild.offline = t; self:UpdateTooltip() end,
					},
					motd = {
						type = 'toggle',
						name = "MotD", desc = "Message of the Day",
						get = function() return self.db.profile.guild.motd end,
						set = function(t) self.db.profile.guild.motd = t; self:Update() end,
					},
					sort = {
						type = 'text',
						name = 'Sort', desc = 'Sort',
						get = function() return self.db.profile.guild.sort end,
						set = function(t) self.db.profile.guild.sort = t; self:Update() end,
						validate = {'name', 'level', 'online', 'zone', 'class', 'rank'},
					},
					columns = makeColumnsGroup('guild'),
					colors = makeColorsGroup('guild'),
				},
			},
			fubar = {
				type = "group",
				name = "FuBar",
				desc = "FuBar options",
				args = AceLibrary("FuBarPlugin-2.0"):GetAceOptionsDataTable(self),
				order = 300,
			},
		},
	}
	friends = {}
	guild = {}
	levels = {}
	classes = {}
	areas = {}
	status = {}
	connected = {}
	ranks = {}
	rankindex = {}
	gnotes = {}
	gonotes = {}
	party = {}
end

function Fu:OnEnable(first)
	self:RegisterEvent("FRIENDLIST_UPDATE", "Update")
	self:SecureHook("FriendsList_Update", "Update")
	self:RegisterEvent("GUILD_ROSTER_UPDATE", "Update")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
end

--[[function Fu:OnMenuRequest()
	dewdrop:InjectAceOptionsAuldLangSyne.menu
	self:AddImpliedMenuOptions()
end--]]

------------------------------------------------------------------------
-- FuBar methods
------------------------------------------------------------------------

function Fu:OnClick()
	ToggleFriendsFrame(1)
end

function Fu:OnDataUpdate()
	friends = clear(friends)
	guild = clear(guild)
	levels = clear(levels)
	classes = clear(classes)
	areas = clear(areas)
	status = clear(status)
	connected = clear(connected)
	ranks = clear(ranks)
	rankindex = clear(rankindex)
	gnotes = clear(gnotes)
	gonotes = clear(gonotes)
	
	if self.db.profile.friends.show then
		self:UpdateFriends()
	end
	if IsInGuild() and self.db.profile.guild.show then
		self:UpdateGuild()
	end
end

function Fu:UpdateFriends()
	local info = AuldLangSyne:HasModule("Info") and AuldLangSyne:GetModule("Info")
	numFriends = GetNumFriends()
	numFriendsOnline = 0
	for i=1, numFriends do
		local name, level, class, zone, fconnected, fstatus = GetFriendInfo(i)
		
		if name then
			table.insert(friends, name)
			if fconnected then
				numFriendsOnline = numFriendsOnline + 1
				levels[name] = level or UNKNOWN
				classes[name] = class or UNKNOWN
				areas[name] = zone or UNKNOWN
				status[name] = fstatus or UNKNOWN
			else
				local thisinfo = info and info.db.realm[name]
				levels[name] = thisinfo and thisinfo.level or UNKNOWN
				classes[name] = thisinfo and thisinfo.class or UNKNOWN
				areas[name] = ''
				status[name] = thisinfo and info:HowLongAgo(thisinfo.updated) or UNKNOWN
			end
			connected[name] = fconnected
		end
	end
	table.sort(friends, Fu.Sort[self.db.profile.friends.sort])
end

function Fu:UpdateGuild()
	numGuild = GetNumGuildMembers(true)
	numGuildOnline = 0
	for i=1, numGuild do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, gstatus = GetGuildRosterInfo(i)
		if name then
			if online then
				numGuildOnline = numGuildOnline + 1
			end
			table.insert(guild, name)
			ranks[name] = rank or UNKNOWN
			rankindex[name] = rankIndex
			levels[name] = level or UNKNOWN
			classes[name] = class or UNKNOWN
			areas[name] = zone or UNKNOWN
			connected[name] = online
			status[name] = gstatus or UNKNOWN
			gnotes[name] = note
			gonotes[name] = officernote
		end
	end
	table.sort(guild, Fu.Sort[self.db.profile.guild.sort])
end

function Fu:PARTY_MEMBERS_CHANGED()
	party = clear(party)
	for i=1,GetNumPartyMembers(),1 do
		party[UnitName("party"..i)] = true
	end
end

function Fu:OnTextUpdate()
	local t
	if self.db.profile.friends.show or self.db.profile.guild.show then
		t = ''
		if self.db.profile.friends.show then
			t = t..numFriendsOnline..'/'..numFriends
		end
		if self.db.profile.friends.show and self.db.profile.guild.show then
			t = t..' '
		end
		if self.db.profile.guild.show then
			t = t..numGuildOnline..'/'..numGuild
		end
	else
		t = 'Auld'
	end
	self:SetText(t)
end

do
	local function editNote(name, type)
		local notes = AuldLangSyne:HasModule("Note") and AuldLangSyne:GetModule("Note")
		if notes then
			if notes.editFrame:IsVisible() and notes.editFrame.name == name then
				notes.editFrame.name = ""
				notes.editFrame.type = ""
				notes.editFrame:Hide()
			else
				notes.editFrame.name = name
				notes.editFrame.type = type
				
				notes.editFrame.editbox:SetText(notes.db.realm[type][name] or '')
				notes.editFrame.edit_text:SetText(format(AceLibrary("AceLocale-2.2"):new("AuldLangSyne_Note")["Editing note for %s"], name))
				notes.editFrame:Show()
			end
		end
	end
	function click(name, type)
		if IsAltKeyDown() then
			InviteUnit(name)
		elseif IsControlKeyDown() then
			editNote(name, type)
		else
			SetItemRef("player:"..name, "|Hplayer:"..name.."|h["..name.."|h", "LeftButton")
		end
	end
	function Fu:OnTooltipUpdate()
		tablet:SetHint("|cffeda55fClick|r to whisper a player. |cffeda55fControl-Click|r to edit a note. |cffeda55fAlt-Click|r to invite to a group.")
		
		local notes = AuldLangSyne:HasModule("Note") and AuldLangSyne:GetModule("Note")
		local fopts = self.db.profile.friends
		if fopts.show then
			local friendcols = new()
			if fopts.columns.name then table.insert(friendcols, 'Name') end
			if fopts.columns.level then table.insert(friendcols, 'Level') end
			if fopts.columns.class then table.insert(friendcols, 'Class') end
			if fopts.columns.zone then table.insert(friendcols, 'Zone') end
			if fopts.columns.status then table.insert(friendcols, 'Status') end
			if fopts.columns.note then table.insert(friendcols, 'Note') end
			local header = new()
			for i,col in ipairs(friendcols) do
				if i==1 then
					header['text'] = col
				else
					header['text'..i] = col
				end
			end
			
			local cat = tablet:AddCategory('id', 'friends', 'columns', #friendcols, 'text', 'Friends', 'justify', 'CENTER')
			cat:AddLine(header)
			local tmp = new()
			for _,name in ipairs(friends) do
				if fopts.offline or connected[name] then
					tmp.name = name
					tmp.level = levels[name]
					tmp.class = classes[name]
					tmp.zone = areas[name]
					tmp.status = status[name]
					if notes and notes.db.realm.friend[name] and notes.db.realm.friend[name] ~= '' then
						local note = notes.db.realm.friend[name]
						if strlen(note) > 50 then
							note = strsub(note, 0, 30)..'...'
						end
						tmp.note = "|cffff8800{"..note.."}|r "
					end
					line = new()
					for i,col in ipairs(friendcols) do
						col = string.lower(col)
						local key, r, g, b
						if i==1 then key = 'text' else key = 'text'..i end
						if fopts.colors[col] == 'class' then
							r,g,b = BC:GetColor(tmp.class)
						elseif fopts.colors[col] == 'level' then
							local c = GetDifficultyColor(tmp.level)
							r,g,b = c.r, c.g, c.b
						else
							if connected[name] then
								r,g,b = 1,1,1
							else
								r,g,b = 0.5,0.5,0.5
							end
						end
						line[key] = tmp[col]
						line[key..'R'], line[key..'G'], line[key..'B'] = r, g, b
					end
					line['func'] = click; line['arg1'] = name; line['arg2'] = 'friend'
					line['hasCheck'] = true
					if party[name] then
						line['checked'] = true
					end
					cat:AddLine(line)
					del(line)
				end
			end
			del(tmp)
			del(header)
			del(friendcols)
		end
		
		local gopts = self.db.profile.guild
		if IsInGuild() and gopts.show then
			local guildcols = new()
			if gopts.columns.name then table.insert(guildcols, 'Name') end
			if gopts.columns.level then table.insert(guildcols, 'Level') end
			if gopts.columns.class then table.insert(guildcols, 'Class') end
			if gopts.columns.zone then table.insert(guildcols, 'Zone') end
			if gopts.columns.rank then table.insert(guildcols, 'Rank') end
			if gopts.columns.status then table.insert(guildcols, 'Status') end
			if gopts.columns.note then table.insert(guildcols, 'Note') end
			local header = new()
			for i,col in ipairs(guildcols) do
				if i==1 then
					header['text'] = col
				else
					header['text'..i] = col
				end
			end
			
			if gopts.motd then
				cat = tablet:AddCategory('id', 'motd', 'text', 'MotD')
				cat:AddLine('text', GetGuildRosterMOTD(), 'wrap', true)
			end
			
			cat = tablet:AddCategory('id', 'guild', 'columns', #guildcols, 'text', 'Guild')
			cat:AddLine(header)
			local tmp = new()
			for _,name in ipairs(guild) do
				if gopts.offline or connected[name] then
					tmp.name = name
					tmp.level = levels[name]
					tmp.class = classes[name]
					tmp.zone = areas[name]
					tmp.status = status[name]
					tmp.rank = ranks[name]
					if gnotes[name] or gonotes[name] then
						local n = ''
						if notes and notes.db.realm.guild[name] and notes.db.realm.guild[name] ~= '' then
							local note = notes.db.realm.guild[name]
							if strlen(note) > 30 then
								note = strsub(note, 0, 30)..'...'
							end
							n = n .. "|cff00aa00{"..note.."}|r "
						end
						if gnotes[name] and gnotes[name] ~= '' then
							n = n .. "|cff00ff00["..gnotes[name].."]|r "
						end
						if gonotes[name] and gonotes[name] ~= '' then
							n = n .. "|cff00ffff["..gonotes[name].."]|r "
						end
						tmp.note = n
					end
					line = new()
					for i,col in ipairs(guildcols) do
						col = string.lower(col)
						local key, r, g, b
						if i==1 then key = 'text' else key = 'text'..i end
						if gopts.colors[col] == 'class' then
							r,g,b = BC:GetColor(tmp.class)
						elseif gopts.colors[col] == 'level' then
							local c = GetDifficultyColor(tmp.level)
							r,g,b = c.r, c.g, c.b
						else
							if connected[name] then
								r,g,b = 1,1,1
							else
								r,g,b = 0.5,0.5,0.5
							end
						end
						line[key] = tmp[col]
						line[key..'R'], line[key..'G'], line[key..'B'] = r, g, b
					end
					line['func'] = click; line['arg1'] = name; line['arg2'] = 'guild'
					line['hasCheck'] = true
					if party[name] then
						line['checked'] = true
					end
					cat:AddLine(line)
					del(line)
				end
			end
			del(tmp)
			del(header)
			del(guildcols)
		end
	end
end

-- Sort methods:

Fu.Sort = {
	name = function(a,b)
		return a < b
	end,
	level = function(a,b)
		if levels[a] == levels[b] then
			return Fu.Sort.class(a,b)
		else
			return levels[a] < levels[b]
		end
	end,
	zone = function(a,b)
		if areas[a] == areas[b] then
			return Fu.Sort.class(a,b)
		else
			return areas[a] < areas[b]
		end
	end,
	class = function(a,b)
		if classes[a] == classes[b] then
			return Fu.Sort.name(a,b)
		else
			return classes[a] < classes[b]
		end
	end,
	rank = function(a,b)
		if rankindex[a] == rankindex[b] then
			return Fu.Sort.name(a,b)
		else
			return rankindex[a] < rankindex[b]
		end
	end,
	online = function(a,b)
		if connected[a] == connected[b] then
			return Fu.Sort.name(a,b)
		else
			return connected[a] --1 or nil, you see.
		end
	end,
}
