local VERSION = tonumber(("$Revision: 41972 $"):match("%d+"))

local _G = getfenv(0)

local AuldLangSyne = AuldLangSyne
local Sync = AuldLangSyne:NewModule("Sync", "AceHook-2.1", "AceConsole-2.0")
if AuldLangSyne.revision < VERSION then
	AuldLangSyne.version = "r" .. VERSION
	AuldLangSyne.revision = VERSION
	AuldLangSyne.date = ("$Date: 2007-06-27 22:35:27 -0400 (Wed, 27 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

function Sync:OnInitialize(name)
	self.db = AuldLangSyne:AcquireDBNamespace("Sync")
	AuldLangSyne:RegisterDefaults("Sync", "char", {
		sync = false,
	})
	AuldLangSyne:RegisterDefaults("Sync", "realm", {
		add_friends = {
			["*"] = {},
		},
		remove_friends = {
			["*"] = {},
		},
	})
	
	self.menu = {
		handler = Sync,
		name = "Sync", type = "toggle",
		desc = "Synchronize the friends lists of all your characters on this realm.",
		get = function() return self.db.char.sync end,
		set = function(t) self.db.char.sync = t; if t then self:SyncFriends() end end,
	}
	
	self.faction = AuldLangSyne.faction
	
	if AuldLangSyne.db.realm.add_friends then
		self.db.realm.add_friends = AuldLangSyne.db.realm.add_friends
		AuldLangSyne.db.realm.add_friends = nil
		self.db.realm.remove_friends = AuldLangSyne.db.realm.remove_friends
		AuldLangSyne.db.realm.remove_friends = nil
		self.db.realm.sync = nil
	end
end

function Sync:OnEnable(first)
	self:SecureHook("AddFriend")
	self:SecureHook("RemoveFriend")
	
	if self.db.char.sync then
		self:SyncFriends()
	end
end

--function Sync:OnDisable()
--	
--end

------------------------------------------------------------------------
-- Friend syncing
------------------------------------------------------------------------

local friendCache = {}
function Sync:SyncFriends()
	local friends = self.db.realm.add_friends[self.faction]
	local remove = self.db.realm.remove_friends[self.faction]
	
	--First, absorb any new friends from this character, and remove any friends scheduled for removal.
	for i=1, GetNumFriends() do
		local name = GetFriendInfo(i)
		if name then
			if remove[name] then
				RemoveFriend(i)
			else
				if not friends[name] then
					friends[name] = true
				end
				friendCache[name] = true
			end
		end
	end
	
	--Now add any friends this character doesn't have yet.
	local i = 1
	for name in pairs(friends) do
		if i > MAX_IGNORE then
			--Complain if we have more than 50 friends in sync. (Using MAX_IGNORE for this, since there isn't a MAX_FRIENDS.)
			self:Print(("More than %d friends are trying to be sync'd.  This won't work.  Make less friends -- what is this, some sort of social game?"):format(MAX_IGNORE))
			break
		end
		if not friendCache[name] then
			AddFriend(name)
		end
		i = i + 1
	end
	
	--And clear that helpful friendCache
	for name in pairs(friendCache) do
		friendCache[name] = nil
	end
end

function Sync:AddFriend(name)
	--Make name unambiguous (it could be capitalized or not, currently)
	name = string.gsub(name, "^(%l)", string.upper, 1)
	if self.db.char.sync and name ~= UnitName("player") then
		self.db.realm.add_friends[self.faction][name] = true
		self.db.realm.remove_friends[self.faction][name] = nil -- just in case
	end
end

function Sync:RemoveFriend(id)
	if type(id) == 'number' then
		id = GetFriendInfo(id)
	end
	if self.db.char.sync and self.db.realm.add_friends[self.faction][id] ~= nil then
		self.db.realm.add_friends[self.faction][id] = nil
		self.db.realm.remove_friends[self.faction][id] = true
	end
end
